------------------------------------------------------
-- AutoCraft.lua
-- Version 1600.1
-- Based on BetterTradeSkill by Matt Perry (http://somewhere.fscked.org)
------------------------------------------------------

local P = {};
AutoCraft = P;

CastingSkill = false;
Unqueueing = false;
OrderedQueue = {};

local MAX_QUEUE_BUTTONS = 10;

function P.OnLoad()
	this:RegisterEvent("CHAT_MSG_SPELL_TRADESKILLS");
	this:RegisterEvent("TRADE_SKILL_SHOW");
	this:RegisterEvent("TRADE_SKILL_CLOSE");
	this:RegisterEvent("SPELLCAST_INTERRUPTED");
	this:RegisterEvent("SPELLCAST_FAILED");
	this:RegisterEvent("SPELLCAST_START");
end

function P.OnEvent(...)
	if (event == "CHAT_MSG_SPELL_TRADESKILLS") then
		for item in string.gfind(arg1, "You create (.+).") do
			P.HandleCreated(item);
			return;
		end
	elseif (event == "TRADE_SKILL_SHOW" or event == "TRADE_SKILL_CLOSE") then
		-- They opened/closed/switched the tradeskill window.  Make sure we clear out the skill queue in case it
		-- was left unempty before.  (This also handles the case of switching to another TradeSkill window... while
		-- it'd be nice to, say, queue some smelting and then some smithing, TradeSkill functions depend on the
		-- window associated with their skill being open.)
		CastingSkill = false;
		OrderedQueue = {};
		P.UpdateDisplay();
	elseif (event == "SPELLCAST_FAILED" or event == "SPELLCAST_INTERRUPTED") then
		CastingSkill = false;
		if (Unqueueing) then
			Unqueueing = false; -- only nuke the rest of the queue for an actual cancel
			P.Iterate();
		else
			OrderedQueue = {};
			P.UpdateDisplay();
		end
	elseif (event == "SPELLCAST_START") then
		CastingSkill = true;
	end
end

function P.Queue_OnClick()
	local id = tonumber(this:GetID());
	if (OrderedQueue[id]) then
		table.remove(OrderedQueue, id);
		if (id == 1) then
			Unqueueing = true;
			SpellStopCasting();
		end
		P.UpdateDisplay();
		P.Queue_UpdateTooltip(id);
	end
end

function P.Queue_OnEvent()
	P.Queue_UpdateTooltip(tonumber(this:GetID()));
end

function P.Queue_UpdateTooltip(id)
	local button = getglobal("QueuedSkillIcon"..id);
	if (button:IsVisible()) then
		GameTooltip:SetOwner(button, "ANCHOR_TOPLEFT");
		if ( id == 1) then
			GameTooltip:SetText(NORMAL_FONT_COLOR_CODE.."Creating: "..FONT_COLOR_CODE_CLOSE, 1.0, 1.0, 1.0);
		else
			GameTooltip:SetText(NORMAL_FONT_COLOR_CODE.."Queued: "..FONT_COLOR_CODE_CLOSE, 1.0, 1.0, 1.0);
		end
		GameTooltip:AppendText(OrderedQueue[id].numQueued .."x ".. OrderedQueue[id].item);
		GameTooltip:AddLine("(Click to cancel)");
		GameTooltip:SetHeight(GameTooltip:GetHeight() + 14);
	end
end

function P.QueueEverything()
	local numSkills = GetNumTradeSkills();
	for index = 1, numSkills do
		local item, amount = P.GetTradeSkillInfo(index);
		if (amount > 0) then
			P.AddToQueue(item, index, amount);
		end
	end
end

function P.Queue(amount)
	local index = GetTradeSkillSelectionIndex();
	local item, maxAmount = P.GetTradeSkillInfo(index);

	if (amount > maxAmount) then amount = maxAmount; end

	if (amount > 0) then
		P.AddToQueue(item, index, amount);
	end
end

function P.AddToQueue(item, index, amount)

	local alreadyQueued = false;
	GFWUtils.Print(GFW_COLOR_START.."Queueing " ..FONT_COLOR_CODE_CLOSE.. amount .. " x " .. GetTradeSkillItemLink(index));
	for _, queueEntry in OrderedQueue do
		if (item == queueEntry.item) then
			queueEntry.numQueued = queueEntry.numQueued + amount;
			alreadyQueued = true;
		end
	end
	if (not alreadyQueued) then
		table.insert(OrderedQueue, {item=item, numQueued=amount, numProduced=0, icon=GetTradeSkillIcon(index)});
	end
	
	P.UpdateDisplay();
	P.Iterate();
end

function P.UpdateDisplay()
	for i = 1, MAX_QUEUE_BUTTONS do
		button = getglobal("QueuedSkillIcon"..i);
		currentItem = OrderedQueue[i];
		if (currentItem == nil) then
			button:Hide();
		else
			SetItemButtonTexture(button, currentItem.icon);
			SetItemButtonCount(button, (currentItem.numQueued - currentItem.numProduced));
			button:Show();
		end
	end
end

function P.Iterate()
	
	if (CastingSkill) then return; end
	if (table.getn(OrderedQueue) > 0) then
		local index = P.GetTradeItemIndex(OrderedQueue[1].item);
		if (index >= 0) then
			DoTradeSkill(index, (OrderedQueue[1].numQueued - OrderedQueue[1].numProduced));
		end
	end
end

function P.HandleCreated(item)
	if (OrderedQueue[1] and item == OrderedQueue[1].item) then
		OrderedQueue[1].numProduced = OrderedQueue[1].numProduced + 1;
		if (OrderedQueue[1].numProduced >= OrderedQueue[1].numQueued) then
			CastingSkill = false; -- this will get set back to true when we get the SPELLCAST_START event
			table.remove(OrderedQueue, 1);
		end
		P.UpdateDisplay();
		P.Iterate();
	end
end

function P.GetTradeSkillInfo(index)
	local skillName, difficulty, amount = GetTradeSkillInfo(index);
	local itemName = P.GetTradeItemName(index);

	return itemName, amount;
end

function P.GetTradeItemName(index)
	local linktext = GetTradeSkillItemLink(index);
	if (linktext) then
		local _,_,item = string.find(linktext, "^.*%[(.*)%].*$");
		return item;
	else
		local item, difficulty, amount = GetTradeSkillInfo(index);
		return item;
	end
end

-- find the trade skill index based on the item name.  this is necessary because sometimes the indices
-- change when your skill level increases.
function P.GetTradeItemIndex(itemName)
	local numSkills = GetNumTradeSkills();
	for index = 1,numSkills do
		local curName = P.GetTradeItemName(index);
		if (curName == itemName) then
			return index;
		end
	end
	return -1;
end
